#include "Parser.h"
#include <iostream>

Parser::Parser(const std::string& input) : myScanner(input) // Scanner initialisieren
{
	// Erstes Token vom Scanner abholen
	getNextToken();
}

int Parser::parse()
{
	// Wir beginnen bei unserem Startsymbol Start
	int res = start();

	// Nun sollten alle Tokens abgearbeitet sein und TT_NIL das Ende der Eingabe besttigen
	accept(TT_NIL);

	return res;
}

// Start = (Multiplikation) { ("+" | "-") (Multiplikation) } ; 
int Parser::start()
{
	// (Multiplikation)
	int res = multiplikation();

	// { ("+" | "-")
	while (myTok.equal(TT_PLUS) || myTok.equal(TT_MINUS))
	{
		switch (myTok.getType())
		{
			case TT_PLUS:
				getNextToken();
				res += multiplikation(); // (Multiplikation)
				break;

			case TT_MINUS:
				getNextToken();
				res -= multiplikation(); // (Multiplikation)
				break;
		}
	}
	// }

	return res;
}

// Multiplikation = (Klammer) { ("*" | "/") (Klammer) } ; 
int Parser::multiplikation()
{
	int res = klammer();

	while (myTok.equal(TT_MUL) || myTok.equal(TT_DIV))
	{
		switch (myTok.getType())
		{
			case TT_MUL:
				getNextToken();
				res *= klammer();
				break;

			case TT_DIV:
				getNextToken();
				res /= klammer();
				break;
		}
	}

	return res;
}

// Klammer = ["+" | "-"] ((Zahl) | "(" (Start) ")") ; 
int Parser::klammer()
{
	int sign = 1; // Fr Vorzeichen (Standard: positives Vorzeichen)

	// Haben wir ein Vorzeichen?
	if (myTok.equal(TT_PLUS) || myTok.equal(TT_MINUS))
	{
		if (myTok.equal(TT_MINUS))
		{
			sign = -1; // Negatives Vorzeichen
		}

		getNextToken();
	}

	// Haben wir eine Klammer?
	if (myTok.equal(TT_LPAREN))
	{
		accept(TT_LPAREN);

		int res = sign * start(); // Rekursion

		accept(TT_RPAREN);

		return res;
	}
	
	// Keine Klammer, also gehen wir von einer Zahl aus
	return sign * zahl();
}

// Zahl = (Ziffer) { (Ziffer) } ; 
// Wir erhalten die gesamte Zahl vom Scanner:
// Es ist also keine Zusammensetzung einzelner Ziffern zu einer Zahl ntig!
int Parser::zahl()
{
	int res = myTok.getValue();

	accept(TT_NUMBER);

	return res;
}

void Parser::accept(TokenType type)
{
	// Stimmt das Token mit dem erwarteten berein?
	if (!myTok.equal(type))
	{
		std::cerr << "Error: unerwartetes Token " << myTok.toString() << ", " << TokenTypeStr[type] << " erwartet" << std::endl;
	}

	// Wir akzeptieren das Token auch im Fehlerfall, um weitere Fehler finden zu knnen
	getNextToken();
}

void Parser::getNextToken()
{
	// Nchstes Token "abholen"
	myTok = myScanner.getNextToken();
}