#ifndef NODE_H
#define NODE_H

#include <sstream>
#include <typeinfo>

// Basisklasse
class Node
{
	public:
		virtual ~Node() { }

		// Ergebnis berechnen
		virtual int eval() const = 0;

		// Codegenerierung
		virtual void generateCode(std::stringstream& strm) = 0;
};

class NodeNumber : public Node
{
	private:
		int myVal;

	public:
		NodeNumber(int val) : myVal(val) { }

		virtual int eval() const { return myVal; }

		virtual void generateCode(std::stringstream& strm)
		{
			strm << myVal ;
		}
};

class NodeAdd : public Node
{
	private:
		Node *myNodeLeft;
		Node *myNodeRight;

	public:
		NodeAdd(Node *left, Node *right) : myNodeLeft(left), myNodeRight(right) { }
		
		virtual ~NodeAdd()
		{
			delete myNodeLeft;
			delete myNodeRight;
		}

		virtual int eval() const { return myNodeLeft->eval() + myNodeRight->eval(); }

		virtual void generateCode(std::stringstream& strm)
		{
			strm << "ADD( ";
			myNodeLeft->generateCode(strm);
			strm << ", ";
			myNodeRight->generateCode(strm);
			strm << " )";
		}
};

class NodeSub : public Node
{
	private:
		Node *myNodeLeft;
		Node *myNodeRight;

	public:
		NodeSub(Node *left, Node *right) : myNodeLeft(left), myNodeRight(right) { }
		
		virtual ~NodeSub()
		{
			delete myNodeLeft;
			delete myNodeRight;
		}

		virtual int eval() const { return myNodeLeft->eval() - myNodeRight->eval(); }

		virtual void generateCode(std::stringstream& strm)
		{
			strm << "SUB( ";
			myNodeLeft->generateCode(strm);
			strm << ", ";
			myNodeRight->generateCode(strm);
			strm << " )";
		}
};

class NodeMul : public Node
{
	private:
		Node *myNodeLeft;
		Node *myNodeRight;

	public:
		NodeMul(Node *left, Node *right) : myNodeLeft(left), myNodeRight(right) { }
		
		virtual ~NodeMul()
		{
			delete myNodeLeft;
			delete myNodeRight;
		}

		virtual int eval() const { return myNodeLeft->eval() * myNodeRight->eval(); }

		virtual void generateCode(std::stringstream& strm)
		{
			if (typeid(*myNodeLeft) == typeid(NodeNumber) && myNodeLeft->eval() == 1)
			{
				myNodeRight->generateCode(strm);
			}
			else
			{

				strm << "MUL( ";
				myNodeLeft->generateCode(strm);
				strm << ", ";
				myNodeRight->generateCode(strm);
				strm << " )";
			}
		}
};

class NodeDiv : public Node
{
	private:
		Node *myNodeLeft;
		Node *myNodeRight;

	public:
		NodeDiv(Node *left, Node *right) : myNodeLeft(left), myNodeRight(right) { }
		
		virtual ~NodeDiv()
		{
			delete myNodeLeft;
			delete myNodeRight;
		}

		virtual int eval() const { return myNodeLeft->eval() / myNodeRight->eval(); }

		virtual void generateCode(std::stringstream& strm)
		{
			strm << "DIV( ";
			myNodeLeft->generateCode(strm);
			strm << ", ";
			myNodeRight->generateCode(strm);
			strm << " )";
		}
};

#endif /* NODE_H */