#include "Scanner.h"
#include <iostream>
#include <cstdlib>

Scanner::Scanner(const std::string& input) : myInput(input), myPos(0)
{
	// Erstes Zeichen einlesen
	readNextChar();
}

Token Scanner::getNextToken()
{
	std::string buf;

	// berflssige Zeichen ignorieren
	skipSpaces();

	switch (myCh)
	{
		case '+':
			readNextChar();
			return Token(TT_PLUS);

		case '-':
			readNextChar();
			return Token(TT_MINUS);

		case '*':
			readNextChar();
			return Token(TT_MUL);

		case '/':
			readNextChar();
			return Token(TT_DIV);

		case '(':
			readNextChar();
			return Token(TT_LPAREN);

		case ')':
			readNextChar();
			return Token(TT_RPAREN);

		case '0': case '1': case '2': case '3': case '4':
		case '5': case '6': case '7': case '8': case '9':
			// Einlesen, solange es eine Ziffer ist
			while (isdigit(myCh))
			{
				buf += myCh;
				readNextChar();
			}

			// Der Einfachheit haben wir die Zahl zunchst in einen String eingelesen
			// Dieser wird nun in ein Integer konvertiert
			return Token(TT_NUMBER, std::atoi(buf.c_str()));

		default:
			if (myCh != 0)
			{
				std::cerr << "Error: nicht unterstuetztes Zeichen '" << myCh << "'" << std::endl;
			}
			break;
	}

	// Es wurde kein passendes Token gefunden. Mgliche Grnde dafr:
	// - Eingabe enthlt kein weiteres Zeichen mehr (Ende erreicht)
	// - Eingabe besteht aus nicht untersttzten Zeichen
	return Token(TT_NIL);
}

void Scanner::skipSpaces()
{
	while (myCh == ' ' || myCh == '\t' || myCh == '\r' || myCh == '\n')
	{
		readNextChar();
	}
}

void Scanner::readNextChar()
{
	// Am Ende der Eingabe angelangt?
	if (myPos > myInput.length())
	{
		myCh = 0;
		return;
	}

	myCh = myInput[myPos++];
}