#include "Parser.h"
#include <iostream>

Parser::Parser(const std::string& input) : myScanner(input) // Scanner initialisieren
{
	// Erstes Token vom Scanner abholen
	getNextToken();
}

Node *Parser::parse()
{
	// Wir beginnen bei unserem Startsymbol Start
	Node *res = start();

	// Nun sollten alle Tokens abgearbeitet sein und TT_NIL das Ende der Eingabe besttigen
	accept(TT_NIL);

	return res;
}

// Start = (Multiplikation) { ("+" | "-") (Multiplikation) } ; 
Node *Parser::start()
{
	Node *res = multiplikation();

	while (myTok.equal(TT_PLUS) || myTok.equal(TT_MINUS))
	{
		switch (myTok.getType())
		{
			case TT_PLUS:
				getNextToken();
				res = new NodeAdd(res, multiplikation());
				break;

			case TT_MINUS:
				getNextToken();
				res = new NodeSub(res, multiplikation());
				break;
		}
	}

	return res;
}

// Multiplikation = (Klammer) { ("*" | "/") (Klammer) } ; 
Node *Parser::multiplikation()
{
	Node *res = klammer();

	while (myTok.equal(TT_MUL) || myTok.equal(TT_DIV))
	{
		switch (myTok.getType())
		{
			case TT_MUL:
				getNextToken();
				res = new NodeMul(res, klammer());
				break;

			case TT_DIV:
				getNextToken();
				res = new NodeDiv(res, klammer());
				break;
		}
	}

	return res;
}

// Klammer = ["+" | "-"] ((Funktion) | (Bezeichner) | (Zahl) | "(" (Start) ")") ;
Node *Parser::klammer()
{
	int sign = 1; // Fr Vorzeichen (Standard: positives Vorzeichen)

	// Haben wir ein Vorzeichen?
	if (myTok.equal(TT_PLUS) || myTok.equal(TT_MINUS))
	{
		if (myTok.equal(TT_MINUS))
		{
			sign = -1; // Negatives Vorzeichen
		}

		getNextToken();
	}

	// Haben wir eine Klammer?
	if (myTok.equal(TT_LPAREN))
	{
		accept(TT_LPAREN);

		Node *res = start(); // Rekursion

		accept(TT_RPAREN);

		return new NodeMul(new NodeNumber(sign), res);
	}
	else if (myTok.equal(TT_IDENT))
	{
		// Ist das folgende Token eine ffnende Klammer?
		if (myScanner.getNextToken(true).equal(TT_LPAREN))
		{
			// Dann ist es eine Funktion
			return funktion();
		}
		
		// Ansonsten eine Konstante
		return bezeichner();
	}
	
	// Keine Klammer, also gehen wir von einer Zahl aus
	return new NodeMul(new NodeNumber(sign), zahl());
}

// Funktion = (Bezeichner) '(' [ (Start) { ',' (Start) } ] ')' ; 
NodeFunc *Parser::funktion()
{
	NodeFunc *res = new NodeFunc(bezeichner());

	accept(TT_LPAREN);

	if (!myTok.equal(TT_RPAREN))
	{
		res->pushArg(start()); // Rekursion

		while (myTok.equal(TT_COMMA))
		{
			accept(TT_COMMA);
			res->pushArg(start()); // Rekursion
		}
	}

	accept(TT_RPAREN);

	return res;
}

// Bezeichner = ( ('a' - 'Z') | '_' ) { ('a' - 'Z') | '_' } ; 
// Wir erhalten den gesamten Bezeichner vom Scanner:
// Es ist hier also keine Zusammensetzung mehr notwendig!
NodeIdent *Parser::bezeichner()
{
	std::string ident = myTok.getStrValue();

	accept(TT_IDENT);

	return new NodeIdent(ident);
}

// Zahl = (Ziffer) { (Ziffer) } ; 
// Wir erhalten die gesamte Zahl vom Scanner:
// Es ist hier also keine Zusammensetzung mehr notwendig!
NodeNumber *Parser::zahl()
{
	double res = myTok.getValue();

	accept(TT_NUMBER);

	return new NodeNumber(res);
}

void Parser::accept(TokenType type)
{
	// Stimmt das Token mit dem erwarteten berein?
	if (!myTok.equal(type))
	{
		std::cerr << "Error: unerwartetes Token " << myTok.toString() << ", " << TokenTypeStr[type] << " erwartet" << std::endl;
	}

	// Wir akzeptieren das Token auch im Fehlerfall, um weitere Fehler finden zu knnen
	getNextToken();
}

void Parser::getNextToken()
{
	// Nchstes Token "abholen"
	myTok = myScanner.getNextToken();
}